#include "RdmaCmProcessor.hpp"
#include <iostream>

RdmaCmProcessor::RdmaCmProcessor(RdmaEndpointGroup *group)
    : _endpointGroup(group)
{
    CPPLog::LOG_INFO("CMProcessor : Step 1 creating event channel\n");
    _eventChannel = rdma_create_event_channel();
    if (_eventChannel == NULL)
    {
       CPPLog::LOG_ERROR( "CMProcesor : error creating event channel\n");
    }
}

struct rdma_cm_id *RdmaCmProcessor::createId()
{
    struct rdma_cm_id *id = NULL;
    int ret = rdma_create_id(_eventChannel, &id, NULL, RDMA_PS_TCP);
    if (ret == -1)
        CPPLog::LOG_ERROR("CMProcesor : rdma_create_id failed\n");
    return id;
}

void RdmaCmProcessor::processCmEvent()
{
    int ret;
    struct rdma_cm_event *event;
    CPPLog::LOG_INFO("CMProcessor : starting cm processing thread\n");
    while (!_stop)
    {
        ret = rdma_get_cm_event(_eventChannel, &event);
        if (ret)
        {
            CPPLog::LOG_ERROR("CMProcesor : rdma_get_cm_event failed\n");
            continue;
        }
        _endpointGroup->processCmEvent(event);
        ret = rdma_ack_cm_event(event);
        if (ret)
        {
            CPPLog::LOG_ERROR("CMProcesor : rdma_ack_cm_event failed\n");
        }
    }
}

void RdmaCmProcessor::start(bool newThread)
{
    if (newThread)
        _cmEventThread = new std::thread(&RdmaCmProcessor::processCmEvent, this);
    else
        processCmEvent();
}

void RdmaCmProcessor::close()
{
    _stop = true;
    if (_cmEventThread != NULL)
        _cmEventThread->join();
    delete _cmEventThread;
    rdma_destroy_event_channel(_eventChannel);
    CPPLog::LOG_ALWAYS("Closed CM Processor");
}
